"""
.. _rcparams_reference:

Matplotlib configuration - rcParams
===================================
Matplotlib's configuration parameters (rcParams) control the behavior and
appearance of plots. These parameters are stored in a dict-like variable
called :data:`matplotlib.rcParams` (also accessible via ``plt.rcParams``).

The following is a comprehensive reference of all available rcParams and their
default values.

.. rcparams::
"""
